MANUAL = "Manual. By button"
LOCAL_1 = "Local. Active Power"
LOCAL_2 = "Local. Current"
LOCAL_3 = "Local. Voltage"
LOCAL_4 = "Local. Magnetic field"
LOCAL_5 = "Local. Current unbalance"
LOCAL_6 = "Local. Temperature"
LOCAL_7 = "Local. Register monitor"
